/*******************************************************************************
 * Copyright (C) 2009
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 *
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 *
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 ******************************************************************************/

package org.n52.security.common.crypto;

import java.io.UnsupportedEncodingException;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.PublicKey;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.KeyGenerator;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;

import org.apache.commons.codec.binary.Base64;

/**
 * Encapsulates some convinient encryption/decryption tasks.
 * <p/>
 * The methods are not intended to be used with big contents! They are not optimized for performance!
 *
 * @author Marko Reiprecht
 */
public final class EncryptionUtil {

    public static final String DEFAULT_SYMMETRIC_ALGORITHM = "AES";
    public static final int DEFAULT_SYMMETRIC_KEYSIZE = 128;
    public static final String DEFAULT_ASYMMETRIC_ALGORITHM = "RSA";
    public static final int DEFAULT_ASYMMETRIC_KEYSIZE = 512;

    public static final int KEY_TYPE_PRIVATE = Cipher.PRIVATE_KEY;
    public static final int KEY_TYPE_PUBLIC = Cipher.PUBLIC_KEY;
    public static final int KEY_TYPE_SECRET = Cipher.SECRET_KEY;

    private EncryptionUtil() {
    }

    /**
     * Resolves the key type from the key.
     *
     * @param key
     *            a key.
     * @return the type of the key.
     */
    public static int getKeyType(final Key key) {
        if (key instanceof SecretKey) {
            return KEY_TYPE_SECRET;
        } else if (key instanceof PrivateKey) {
            return KEY_TYPE_PRIVATE;
        } else if (key instanceof PublicKey) {
            return KEY_TYPE_PUBLIC;
        } else {
            throw new IllegalArgumentException("unknown key type");
        }
    }

    /**
     * Encrypts the contents of the input string, with the given key and the given algorithm.
     * <p/>
     * The resulting bytes of the encryption are codes with BASE64 and returned as string.
     *
     * @param content
     *            input.
     * @param key
     *            the key to use.
     * @param transformation
     *            the transformation to use.
     * @return the encrypted bytes as base64 string.
     */
    public static String encrypt(final String content, final Key key, final String transformation) {
        try {
            Cipher cipher = Cipher.getInstance(transformation);
            cipher.init(Cipher.ENCRYPT_MODE, key);
            byte[] cipherBytes = cipher.doFinal(content.getBytes("UTF-8"));
            byte[] base64bytes = Base64.encodeBase64(cipherBytes, false);
            return new String(base64bytes, "UTF-8");
        } catch (NoSuchAlgorithmException e) {
            throw new IllegalArgumentException("transformation '" + transformation + "'is unknown: " + e, e);
        } catch (NoSuchPaddingException e) {
            throw new IllegalArgumentException("wrong padding value in the transformation '" + transformation + "' : "
                    + e, e);
        } catch (InvalidKeyException e) {
            throw new IllegalArgumentException("invalid key '" + key.getAlgorithm() + "|" + key.getFormat() + "' :"
                    + e, e);
        } catch (IllegalBlockSizeException e) {
            throw new IllegalArgumentException("illegale block size :"
                    + e, e);
        } catch (BadPaddingException e) {
            throw new IllegalArgumentException("bad padding :"
                    + e, e);
        } catch (UnsupportedEncodingException e) {
            throw new IllegalArgumentException("invalid enconding :"
                    + e, e);
        }
    }

    /**
     * Decryptes the given result of a previous encrypt (base64 byte string) with the given key and using the given
     * algorithm.
     *
     * @param encryptedBytesAsBase64
     *            the encryption bytes.
     * @param key
     *            the key to decrypt.
     * @param transformation
     *            the algorithm to use.
     * @return the decrypted string.
     */
    public static String decrypt(final String encryptedBytesAsBase64, final Key key, final String transformation) {
        try {
            Cipher cipher = Cipher.getInstance(transformation);
            cipher.init(Cipher.DECRYPT_MODE, key);
            byte[] cipherBytes = Base64.decodeBase64(encryptedBytesAsBase64.getBytes("UTF-8"));
            byte[] decrpytedBytes = cipher.doFinal(cipherBytes);
            return new String(decrpytedBytes, "UTF-8");
        } catch (NoSuchAlgorithmException e) {
            throw new IllegalArgumentException("transformation '" + transformation + "'is unknown: " + e, e);
        } catch (NoSuchPaddingException e) {
            throw new IllegalArgumentException("wrong padding value in the transformation '" + transformation + "' : "
                    + e, e);
        } catch (InvalidKeyException e) {
            throw new IllegalArgumentException("invalid key '" + key.getAlgorithm() + "|" + key.getFormat() + "' :"
                    + e, e);
        } catch (IllegalBlockSizeException e) {
            throw new IllegalArgumentException("illegale block size :"
                    + e, e);
        } catch (BadPaddingException e) {
            throw new IllegalArgumentException("bad padding :"
                    + e, e);
        } catch (UnsupportedEncodingException e) {
            throw new IllegalArgumentException("invalid enconding :"
                    + e, e);
        }
    }

    /**
     * Wraps(encrypts) a secret key to securly transport it over a network.
     *
     * @param toWrap
     *            the key to wrap.
     * @param key
     *            the key used in the wrapping encryption.
     * @param transformation
     *            the algorithm.
     * @return the bytes of the wrapped key as base64 string.
     */
    public static String wrapKey(final Key toWrap, final Key key, final String transformation) {
        try {
            Cipher cipher = Cipher.getInstance(transformation);
            cipher.init(Cipher.WRAP_MODE, key);
            byte[] cipherBytes = cipher.wrap(toWrap);
            byte[] base64bytes = Base64.encodeBase64(cipherBytes, false);
            return new String(base64bytes, "UTF-8");
        } catch (NoSuchAlgorithmException e) {
            throw new IllegalArgumentException("transformation '" + transformation + "'is unknown: " + e, e);
        } catch (NoSuchPaddingException e) {
            throw new IllegalArgumentException("wrong padding value in the transformation '" + transformation + "' : "
                    + e, e);
        } catch (InvalidKeyException e) {
            throw new IllegalArgumentException("invalid key '" + key.getAlgorithm() + "|" + key.getFormat() + "' :"
                    + e, e);
        } catch (IllegalBlockSizeException e) {
            throw new IllegalArgumentException("illegale block size :"
                    + e, e);
        } catch (UnsupportedEncodingException e) {
            throw new IllegalArgumentException("invalid enconding :"
                    + e, e);
        }
    }

/**
     * Unwraps a secret key, next to the base64 bytes of the wrapped key you need to know the algorithm used in the key
     * and the type of the key (e.g. {@link #KEY_TYPE_SECRET).
     *
     * @param encryptedKeyBytesAsBase64
     *            the string produced by the wrap method.
     * @param encryptedKeyAlgorithm
     *            the algorithm of the encrypted key.
     * @param encryptedKeyType
     *            the type of the encrypted Key (e.g. {@link #KEY_TYPE_SECRET)).
     * @param key
     *            the key used to decrypt.
     * @param transformation
     *            the algorithm used to decrypt the key.
     * @return a decrypted key.
     */
    public static Key unwrapKey(final String encryptedKeyBytesAsBase64,
                                final String encryptedKeyAlgorithm,
                                final int encryptedKeyType,
                                final Key key,
                                final String transformation) {
        try {
            Cipher cipher = Cipher.getInstance(transformation);
            cipher.init(Cipher.UNWRAP_MODE, key);
            byte[] cipherBytes = Base64.decodeBase64(encryptedKeyBytesAsBase64.getBytes("UTF-8"));
            return cipher.unwrap(cipherBytes, encryptedKeyAlgorithm, encryptedKeyType);
        } catch (NoSuchAlgorithmException e) {
            throw new IllegalArgumentException("transformation '" + transformation + "'is unknown: " + e, e);
        } catch (NoSuchPaddingException e) {
            throw new IllegalArgumentException("wrong padding value in the transformation '" + transformation + "' : "
                    + e, e);
        } catch (InvalidKeyException e) {
            throw new IllegalArgumentException("invalid key '" + key.getAlgorithm() + "|" + key.getFormat() + "' :"
                    + e, e);
        } catch (UnsupportedEncodingException e) {
            throw new IllegalArgumentException("invalid enconding :"
                    + e, e);
        }
    }

    /**
     * Default method for symmetric encryption using the AES algorithm.
     *
     * @param content
     *            the content to encrypt.
     * @param symmetricKey
     *            the symmetric key to use.
     * @return base 64 encoded encrypted bytes.
     */
    public static String encryptSymmetric(final String content, final Key symmetricKey) {
        return encrypt(content, symmetricKey, DEFAULT_SYMMETRIC_ALGORITHM);
    }

    /**
     * Default method for symmetric decryption using the AES algorithm.
     *
     * @param encryptedBytesAsBase64
     *            base64 encrypted bytes.
     * @param symmetricKey
     *            the symmetric key.
     * @return the plain content.
     */
    public static String decryptSymmetric(final String encryptedBytesAsBase64, final Key symmetricKey) {
        return decrypt(encryptedBytesAsBase64, symmetricKey, DEFAULT_SYMMETRIC_ALGORITHM);
    }

    /**
     * Default method for symmetric key wrapping using the AES algorithm.
     *
     * @param toWrap
     *            the Key to encrypt.
     * @param symmetricKey
     *            the symmetric key to use.
     * @return base 64 encoded encrypted bytes.
     */
    public static String wrapSymmetric(final Key toWrap, final Key symmetricKey) {
        return wrapKey(toWrap, symmetricKey, DEFAULT_SYMMETRIC_ALGORITHM);
    }

/** Default method for symmetric unwrapping using the AES algorithm.
     *
     * @param encryptedKeyBytesAsBase64
     *            the string produced by the wrap method.
     * @param encryptedKeyAlgorithm
     *            the algorithm of the encrypted key.
     * @param encryptedKeyType
     *            the type of the encrypted Key (e.g. {@link #KEY_TYPE_SECRET)).
     * @param symmetricKey
     *            the symmetric key.
     * @return the unwraped key.
     */
    public static Key unwrapSymmetric(final String encryptedKeyBytesAsBase64,
                                      final String encryptedKeyAlgorithm,
                                      final int encryptedKeyType,
                                      final Key symmetricKey) {
        return unwrapKey(encryptedKeyBytesAsBase64, encryptedKeyAlgorithm, encryptedKeyType, symmetricKey,
                DEFAULT_SYMMETRIC_ALGORITHM);
    }

    /**
     * Default method for symmetric encryption using the RSA algorithm.
     *
     * @param content
     *            the content to encrypt.
     * @param asymmetricKey
     *            the key to use.
     * @return base 64 encoded encrypted bytes.
     */
    public static String encryptAsymmetric(final String content, final Key asymmetricKey) {
        return encrypt(content, asymmetricKey, DEFAULT_ASYMMETRIC_ALGORITHM);
    }

    /**
     * Default method for asymmetric decryption using the RSA algorithm.
     *
     * @param encryptedBytesAsBase64
     *            base64 encrypted bytes.
     * @param asymmetricKey
     *            the asymmetric key.
     * @return the plain content.
     */
    public static String decryptAsymmetric(final String encryptedBytesAsBase64, final Key asymmetricKey) {
        return decrypt(encryptedBytesAsBase64, asymmetricKey, DEFAULT_ASYMMETRIC_ALGORITHM);
    }

    /**
     * Default method for asymmetric key wrapping using the AES algorithm.
     *
     * @param toWrap
     *            the Key to encrypt.
     * @param asymmetricKey
     *            the asymmetric key to use.
     * @return base 64 encoded encrypted bytes.
     */
    public static String wrapAsymmetric(final Key toWrap, final Key asymmetricKey) {
        return wrapKey(toWrap, asymmetricKey, DEFAULT_ASYMMETRIC_ALGORITHM);
    }

/** Default method for asymmetric unwrapping using the AES algorithm.
     *
     * @param encryptedKeyBytesAsBase64
     *            the string produced by the wrap method.
     * @param encryptedKeyAlgorithm
     *            the algorithm of the encrypted key.
     * @param encryptedKeyType
     *            the type of the encrypted Key (e.g. {@link #KEY_TYPE_SECRET)).
     * @param asymmetricKey
     *            the asymmetric key.
     * @return the unwraped key.
     */
    public static Key unwrapAsymmetric(final String encryptedKeyBytesAsBase64,
                                       final String encryptedKeyAlgorithm,
                                       final int encryptedKeyType,
                                       final Key asymmetricKey) {
        return unwrapKey(encryptedKeyBytesAsBase64, encryptedKeyAlgorithm, encryptedKeyType, asymmetricKey,
                DEFAULT_ASYMMETRIC_ALGORITHM);
    }

    /**
     * Creates a KeyPair with default settings (RSA 512).
     *
     * @return asymmetric keypair.
     */
    public static KeyPair createAsymmetricKeys() {
        return createAsymmetricKeys(DEFAULT_ASYMMETRIC_KEYSIZE, DEFAULT_ASYMMETRIC_ALGORITHM);
    }

    /**
     * Creates KeyPair with default algorithm (RSA) but given size.
     *
     * @param size
     *            key size.
     * @return a new KeyPair.
     */
    public static KeyPair createAsymmetricKeys(final int size) {
        return createAsymmetricKeys(size, DEFAULT_ASYMMETRIC_ALGORITHM);
    }

    /**
     * Creates new key pair with given algorithm and size.
     *
     * @param size
     *            key size.
     * @param algorithm
     *            algorithm.
     * @return new keypair.
     */
    public static KeyPair createAsymmetricKeys(final int size, final String algorithm) {
        try {
            KeyPairGenerator keyPairgen = KeyPairGenerator.getInstance(algorithm);
            keyPairgen.initialize(size);
            return keyPairgen.genKeyPair();
        } catch (NoSuchAlgorithmException ex) {
            throw new IllegalArgumentException("key pair generator algorithm <" + algorithm + "> not known");
        }
    }

    /**
     * creates symmetric key with default algorithm (AES 128).
     *
     * @return new key.
     */
    public static Key createSymmetricKey() {
        return createSymmetricKey(DEFAULT_SYMMETRIC_KEYSIZE, DEFAULT_SYMMETRIC_ALGORITHM);
    }

    /**
     * creates symmetric key with default algorithm (AES) but different key size.
     *
     * @return new key.
     */
    public static Key createSymmetricKey(final int size) {
        return createSymmetricKey(size, DEFAULT_SYMMETRIC_ALGORITHM);
    }

    /**
     * creates symmetric key with given algorithm and key size.
     *
     * @return new key.
     */
    public static Key createSymmetricKey(final int size, final String algorithm) {
        try {
            KeyGenerator keygen = KeyGenerator.getInstance(algorithm);
            keygen.init(size);
            return keygen.generateKey();
        } catch (NoSuchAlgorithmException ex) {
            throw new IllegalArgumentException("key pair generator algorithm <" + algorithm + "> not known");
        }
    }

}
