package org.n52.security.common.subject;

import java.security.Principal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.security.auth.Subject;

import org.n52.security.common.attributes.Attribute;
import org.n52.security.common.attributes.AttributeValue;

public class SubjectPrincipalAnalyzer {

    private final Subject m_subject;

    public SubjectPrincipalAnalyzer(Subject subject) {
        m_subject = subject;
    }

    /**
     * Retrieves the value of the first {@link AttributePrincipal} with a given name.
     * 
     * @param subject
     *            subject containing the attribute information
     * @param attributeName
     *            the name of the attribute that contains the information.
     * @return the String value of the first {@link AttributePrincipal} of the subject
     *         where {@link AttributePrincipal#getName()} equals <code>attributeName</code>, <code>null</code> if no
     *         such exists.
     */
    public String getSingleAttributeValue(final String attributeName) {
        AttributePrincipal attributePrincipal = findPrincipal(AttributePrincipal.class, attributeName);
        return attributePrincipal != null ? attributePrincipal.getValue() : null;
    }

    /**
     * Retrieves the {@link AttributeValue} of the first {@link AttributePrincipal} with a given name.
     * 
     * @param subject
     *            subject containing the attribute information
     * @param attributeName
     *            the name of the attribute that contains the information.
     * @return the String value of the first {@link AttributePrincipal} of the subject
     *         where {@link AttributePrincipal#getName()} equals <code>attributeName</code>, <code>null</code> if no
     *         such exists.
     */
    public AttributeValue getComplexAttributeValue(String attributeName) {
        AttributePrincipal attributePrincipal = findPrincipal(AttributePrincipal.class, attributeName);
        return attributePrincipal != null ? attributePrincipal.getAttributeValue() : null;
    }

    /**
     * Creates a map of all attributes with (the last) values as string.
     * 
     * @return Map<String,String>
     */
    public Map<String, String> getSimpleAttributeValues() {
        Map<String, String> attributes = new HashMap<String, String>();
        for (AttributePrincipal attributePrincipal : getPrincipals(AttributePrincipal.class)) {
            attributes.put(attributePrincipal.getName(), attributePrincipal.getValue());
        }
        return attributes;
    }

    /**
     * Creates a map of all attributes with (the last) values as {@link AttributeValue}.
     * 
     * @return Map<String,{@link AttributeValue}>
     */
    public Map<String, AttributeValue> getComplexAttributeValues(final Subject subject) {
        Map<String, AttributeValue> attributes = new HashMap<String, AttributeValue>();
        for (AttributePrincipal attributePrincipal : getPrincipals(AttributePrincipal.class)) {
            attributes.put(attributePrincipal.getName(), attributePrincipal.getAttributeValue());
        }
        return attributes;
    }

    /**
     * Creates a list of {@link Attribute}s within the subject.
     * 
     * @param subject
     * @return
     */
    public List<Attribute> getAttributes() {
        List<Attribute> attributes = new ArrayList<Attribute>();
        for (AttributePrincipal attributePrincipal : getPrincipals(AttributePrincipal.class)) {
            attributes.add(attributePrincipal.getAttribute());
        }
        return attributes;
    }

    /**
     * Retrieves all RolePrincipal names form the subject.
     * 
     * @param subject
     *            the current subject.
     * @return
     */
    public String[] getRolesAsArray() {
        Set<String> roles = getRoles();
        return roles.toArray(new String[roles.size()]);
    }

    /**
     * Retrieves all RolePrincipal names form the subject.
     * 
     * @param subject
     *            the current subject.
     * @return
     */
    public Set<String> getRoles() {
        Set<String> roles = new HashSet<String>();
        for (RolePrincipal role : getPrincipals(RolePrincipal.class)) {
            roles.add(role.getName());
        }
        return roles;
    }

    /**
     * Responsible for checking if a user has a given role.
     * 
     * @param subject
     *            the current subject.
     * @param role
     *            a role name.
     * @return
     */
    public boolean isInRole(final String role) {
        return findPrincipal(RolePrincipal.class, role) != null;
    }

    /**
     * Retrieves the username of the subject.
     * 
     * @param subject
     *            containing the username information.
     * @return the name of the first {@link UsernameIDPrincipal} found, <code>null</code> otherwise.
     */
    public String getUsername() {
        UsernameIDPrincipal usernameIDPrincipal = getPrincipal(UsernameIDPrincipal.class);
        return usernameIDPrincipal == null ? null : usernameIDPrincipal.getName();
    }

    /**
     * Gets fist principal of given class with given name.
     */
    public <T extends Principal> T findPrincipal(Class<T> principalClass, String name) {
        for (T principal : getPrincipals(principalClass)) {
            if (name.equals(principal.getName())) {
                return principal;
            }
        }
        return null;
    }

    /**
     * Gets first principal of given class.
     */
    public <T extends Principal> T getPrincipal(Class<T> principalClass) {
        return getFirstItem(m_subject.getPrincipals(principalClass));
    }

    public <T extends Principal> Set<T> getPrincipals(Class<T> principalClass) {
        return m_subject.getPrincipals(principalClass);
    }

    public void removePrincipal(Principal principal) {
        m_subject.getPrincipals().remove(principal);
    }

    private <T> T getFirstItem(Set<T> setOfItems) {
        if (setOfItems.isEmpty()) {
            return null;
        }
        return setOfItems.iterator().next();
    }

    public Subject getSubject() {
        return m_subject;
    }

}
