/*******************************************************************************
 * Copyright (C) 2008
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 *
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 *
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 *
 * Author: Marko Reiprecht
 * Created: 24.07.2008
 * Modified: $Date $
 ******************************************************************************/
/* $Id: $ */
package org.n52.security.common.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;

/**
 * Class FileFinder is a helper class to encapsulate the typical finding of a
 * file.
 * <p/>
 * todo: maybe we can align the resource concept of spring to such tasks.
 *
 * @author Marko Reiprecht Created on 24.07.2008
 * @version $Revision: $
 */
public class FileFinder {

    // ------------------------------ FIELDS ------------------------------

    private String m_fileName;

    // --------------------------- CONSTRUCTORS ---------------------------

    /**
     * Constructor creates a new instance.
     */
    public FileFinder() {
    }

    /**
     * Constructor creates a new instance.
     *
     * @param fileName
     *            the fileName.
     */
    public FileFinder(final String fileName) {
        m_fileName = fileName;
    }

    // --------------------- GETTER / SETTER METHODS ---------------------

    /**
     * Method gets the fileName.
     *
     * @return String.
     */
    public String getFileName() {
        if (m_fileName == null) {
            throw new IllegalStateException("property <fileName> not configured properly");
        }
        return m_fileName;
    }

    /**
     * Method sets the fileName.
     *
     * @param fileName
     *            the fileName.
     */
    public void setFileName(final String fileName) {
        m_fileName = fileName;
    }

    /**
     * Method gets the inputStream for a given file name.
     * <p/>
     * Works not with directories. (self explaining or). But it is able to get
     * strems from files within jars.
     *
     * @return InputStream.
     */
    public InputStream getInputStream() {
        // 1. check per resource loading
        String filename = getFileName();
        InputStream fileIn = getClass().getResourceAsStream("/" + filename);
        if (fileIn != null) {
            return fileIn;
        }
        if (filename.startsWith("WEB-INF") || filename.startsWith("/WEB-INF")) {
            fileIn =
                    getClass().getClassLoader().getResourceAsStream(
                            filename.replaceFirst("/?WEB-INF", "/../../WEB-INF"));
            if (fileIn != null) {
                return fileIn;
            }
            String help = getClass().getClassLoader().getResource("/").toExternalForm();
            help = help.replace("WEB-INF/classes/", "");
            help += filename;
            try {
                fileIn = new URL(help).openStream();
            } catch (MalformedURLException e) {
                throw new RuntimeException("Unable to read file: " + help, e);
            } catch (IOException e) {
                throw new RuntimeException("Unable to open stream for file: " + help, e);
            }
            if (fileIn != null) {
                return fileIn;
            }
        }

        // 2. check per File class
        File file = new File(filename);
        if (file.exists() && file.isFile()) {
            try {
                return new FileInputStream(file);
            } catch (FileNotFoundException e) {
                throw new IllegalStateException("file not found, but exists: " + e);
            }
        }
        throw new IllegalArgumentException("File <" + filename + "> not found");
    }

    /**
     * Method gets the file for a given file name.
     * <p/>
     * It checks the existence of the file, but this can be a directory, too.
     *
     * @return File.
     */
    public File getFile() {
        // 1. check per resource loading
        String filename = getFileName();
        File file = null;
        URL fileIn = getClass().getResource("/" + filename);
        if (fileIn == null && (filename.startsWith("WEB-INF") || filename.startsWith("/WEB-INF"))) {
            fileIn = getClass().getClassLoader().getResource(filename.replaceFirst("/?WEB-INF", "/../../WEB-INF"));
        }
        if (fileIn != null) {
            try {
                file = new File(URI.create(fileIn.toExternalForm()));
            } catch (IllegalArgumentException e) {
                // TODO Only for ServletExec???
                file = new File(fileIn.getFile());
            }
        }
        if (file == null) {
            // check per File class
            file = new File(filename);
        }
        if (file.exists()) {
            return file;
        }
        throw new IllegalArgumentException("File <" + filename + "> not found");
    }
}
