/*******************************************************************************
 * Copyright (C) 2008
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 *
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 *
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 *
 * Author: Marko Reiprecht
 * Created: 22.03.2008
 ******************************************************************************/
/* $Id: $ */
package org.n52.security.common.xml;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * Class QuietErrorHandler collects all errors thrown by a parser, but never rethrow it.
 * <p/>
 * A client can use one of the is* and get* methods to retrieve the thrown warnings, errors and fatal errors.
 * <p/>
 * The handler also logs the errors, but only on debug level, because a client shall evaluate the errors and properly
 * react on them.
 * 
 * @author Marko Reiprecht
 *         Created on 22.03.2008
 * @version $Revision: $
 */
public class QuietErrorHandler implements ErrorHandler {

    // ------------------------------ FIELDS ------------------------------

    /**
     * The Logger.
     */
    private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(QuietErrorHandler.class);

    private List<Exception> m_errors = new ArrayList<Exception>();
    private List<Exception> m_warnings = new ArrayList<Exception>();
    private List<Exception> m_fatalErrors = new ArrayList<Exception>();

    // --------------------- GETTER / SETTER METHODS ---------------------

    /**
     * Method gets the errors.
     * 
     * @return Iterator.
     */
    public Iterator getErrors() {
        return m_errors.iterator();
    }

    /**
     * Method gets the warnings.
     * 
     * @return Iterator.
     */
    public Iterator getWarnings() {
        return m_warnings.iterator();
    }

    /**
     * Method checks if warnings, errors or fatal errors are collected.
     * 
     * @return boolean.
     */
    public boolean isEmpty() {
        return !isError() && !isWarning() && !isFatalErrors();
    }

    /**
     * Method checks if errors are collected.
     * 
     * @return boolean.
     */
    public boolean isError() {
        return !m_errors.isEmpty();
    }

    /**
     * Method checks if fatal errors are collected.
     * 
     * @return boolean.
     */
    public boolean isFatalErrors() {
        return !m_fatalErrors.isEmpty();
    }

    /**
     * Method gets the fatalErrors.
     * 
     * @return Iterator.
     */
    public Iterator getFatalErrors() {
        return m_fatalErrors.iterator();
    }

    /**
     * Checks if warnings are collected.
     * 
     * @return boolean.
     */
    public boolean isWarning() {
        return !m_warnings.isEmpty();
    }

    // ------------------------ INTERFACE METHODS ------------------------

    // --------------------- Interface ErrorHandler ---------------------

    /**
     * Handles the errors.
     * 
     * @param exception
     *            the exception.
     * @throws SAXException
     *             if error occurs.
     */
    public void error(final SAXParseException exception) throws SAXException {
        if (LOG.isDebugEnabled()) {
            log("error", exception);
        }
        m_errors.add(exception);
    }

    /**
     * Handles the fatal errors.
     * 
     * @param exception
     *            the exception.
     * @throws SAXException
     *             if error occurs.
     */
    public void fatalError(final SAXParseException exception) throws SAXException {
        if (LOG.isDebugEnabled()) {
            log("fatal error", exception);
        }
        m_fatalErrors.add(exception);
    }

    /**
     * Handles the warning exceptions.
     * 
     * @param exception
     *            the exception.
     * @throws SAXException
     *             if error occurs.
     */
    public void warning(final SAXParseException exception) throws SAXException {
        if (LOG.isDebugEnabled()) {
            log("warning", exception);
        }
        m_warnings.add(exception);
    }

    // -------------------------- OTHER METHODS --------------------------

    /**
     * Method logs the exception.
     * 
     * @param level
     *            the level.
     * @param exception
     *            the exception.
     */
    private void log(final String level, final SAXParseException exception) {
        LOG.debug(new StringBuffer()
                .append("SAX parser ").append(level)
                .append(" in line: ").append(exception.getLineNumber())
                .append(" column: ").append(exception.getColumnNumber())
                .append(" publicId: ").append(exception.getPublicId())
                .append(" systemId: ").append(exception.getSystemId())
                .append(" msg: ").append(exception.getMessage()).toString());
    }

    /**
     * Method resets the internal state, so it is possible to use an instance many times.
     */
    public void reset() {
        m_warnings.clear();
        m_errors.clear();
        m_fatalErrors.clear();
    }
}
