/*******************************************************************************
 * Copyright (C) 2008
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 *
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 *
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 *
 * Author: Marko Reiprecht
 * Created: 18.04.2008
 ******************************************************************************/
package org.n52.security.common.xml;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamException;

import org.xml.sax.InputSource;

/**
 * Class XMLStreamChunkReader reads an xml document with an internal xml stream reader
 * and as chunks and calls a subsets of XMLStreamChunkHandler instances.
 * <p/>
 * Xml elements for which no handler is defined are simply skipped.
 * 
 * @author Marko Reiprecht
 *         Created on 18.04.2008
 * @version $Revision: $
 */
public class XMLChunkReader {

    /**
     * Contains a map from qnames to handlers.
     */
    private Map<QName, XMLChunkHandler> m_partialStreamHandler = new HashMap<QName, XMLChunkHandler>();

    /**
     * Method registers a PartialStreamHandler for a Qname.
     * 
     * @param qname
     *            the qname.
     * @param handler
     *            the handler.
     * @return this.
     */
    public XMLChunkReader addHandler(final QName qname, final XMLChunkHandler handler) {
        m_partialStreamHandler.put(qname, handler);
        return this;
    }

    /**
     * Method gets the Handler for the qname.
     * 
     * @param qname
     *            the qname.
     * @return XmlPartialStreamHandler.
     */
    public XMLChunkHandler getHandler(final QName qname) {
        return (XMLChunkHandler) m_partialStreamHandler.get(qname);
    }

    /**
     * Method reads an xml source.
     * 
     * @param source
     *            the source.
     * @throws XMLStreamException
     *             if error occurs.
     */
    public void read(final InputSource source) {
        Set<QName> qnames = m_partialStreamHandler.keySet();
        XMLChunkIterator chunkIterator = new XMLChunkIterator(source, qnames.toArray(new QName[qnames.size()]));
        while (chunkIterator.hasNext()) {
            XMLChunk chunk = chunkIterator.next();
            if (chunk.isStart()) {
                handleChunkStart(chunk);
            } else {
                handleChunkEnd(chunk);
            }
        }
    }

    /**
     * Handles a chunk start.
     */
    private void handleChunkStart(final XMLChunk chunk) {
        List<QName> xpath = Collections.unmodifiableList(chunk.getXpath());
        XMLChunkHandler handler = getHandler(chunk.getStartElement());

        if (chunk.hasParent()) {
            XMLChunk parentChunk = chunk.getParent();
            XMLChunkHandler parentHandler = getHandler(parentChunk.getStartElement());
            parentHandler.handleChunkBeforChildHandler(
                    xpath,
                    parentChunk.getStartElement(),
                    parentChunk.getChunk(),
                    chunk.getStartElement(),
                    handler);
        }
        handler.handleChunkStart(xpath, chunk.getStartElement());
    }

    /**
     * Handler function of the end chunk event.
     */
    private void handleChunkEnd(final XMLChunk chunk) {

        List<QName> xpath = Collections.unmodifiableList(chunk.getXpath());
        XMLChunkHandler handler = getHandler(chunk.getStartElement());
        handler.handleChunkEnd(xpath, chunk.getStartElement(), chunk.getChunk());

        if (chunk.hasParent()) {
            XMLChunk parentChunk = chunk.getParent();
            XMLChunkHandler parentHandler = getHandler(parentChunk.getStartElement());
            parentHandler.handleChunkAfterChildHandler(
                    xpath,
                    parentChunk.getStartElement(),
                    parentChunk.getChunk(),
                    chunk.getStartElement(),
                    handler);
        }

    }

}
